@@ -526,8 +526,16 @@ SimRules::SimRules( int fd, QObject *p,  const QString& filename, HardwareManipu
              this, SLOT(dialCheck(QString,bool&)) );
 
     if ( machine ) {
-        connect( machine, SIGNAL(startIncomingCall(QString)),
-                 _callManager, SLOT(startIncomingCall(QString)) );
+        connect( machine, SIGNAL(startIncomingCall(QString,QString,QString)),
+                 _callManager, SLOT(startIncomingCall(QString,QString,QString)) );
+        connect ( _callManager, SIGNAL( callStatesChanged( QList<CallInfo> * ) ),
+                  machine, SLOT( callManagement( QList<CallInfo> * ) ) );
+        connect ( machine, SIGNAL( stateChangedToAlerting() ), _callManager,
+                SLOT( dialingToAlerting() ) );
+        connect ( machine, SIGNAL( stateChangedToConnected() ), _callManager,
+                SLOT( dialingToConnected() ) );
+        connect ( machine, SIGNAL( stateChangedToHangup( int ) ), _callManager,
+                SLOT( hangupRemote( int ) ) );
     }
 
     connect(this,SIGNAL(readyRead()),
@@ -537,8 +545,6 @@ SimRules::SimRules( int fd, QObject *p,  const QString& filename, HardwareManipu
     // Initialize the local state.
     currentState = 0;
     defState = 0;
-    return_error_string = "";
-    return_error_count = 0;
     usedCallIds = 0;
     fileSystem = 0;
     useGsm0710 = false;
@@ -546,8 +552,13 @@ SimRules::SimRules( int fd, QObject *p,  const QString& filename, HardwareManipu
     incomingUsed = 0;
     lineUsed = 0;
     defaultToolkitApp = toolkitApp = new DemoSimApplication( this, this );
+    conformanceApp = new ConformanceSimApplication( this, this );
     connect( _callManager, SIGNAL(controlEvent(QSimControlEvent)),
              toolkitApp, SLOT(controlEvent(QSimControlEvent)) );
+
+    simApps.append( toolkitApp );
+    simApps.append( conformanceApp );
+
     if ( machine )
         machine->handleNewApp();
 
@@ -839,9 +850,15 @@ void SimRules::tryReadCommand()
 
 void SimRules::destruct()
 {
-    if ( toolkitApp != defaultToolkitApp )
-        delete toolkitApp;
+    int count = simApps.count();
+
+    for ( int i = 0; i < count; i++ )
+        simApps.removeAt( 0 );
+
+    delete conformanceApp;
+    conformanceApp = NULL;
     delete defaultToolkitApp;
+    defaultToolkitApp = NULL;
     toolkitApp = NULL;
 
     if ( getMachine() )
@@ -877,13 +894,18 @@ HardwareManipulator * SimRules::getMachine() const
 
 void SimRules::setSimApplication( SimApplication *app )
 {
-    if ( toolkitApp != defaultToolkitApp )
-        delete toolkitApp;
-    toolkitApp = ( app ? app : defaultToolkitApp );
-    toolkitApp->start();
+    if ( toolkitApp == app )
+        return;
 
-    if ( getMachine() )
-        getMachine()->handleNewApp();
+    if ( toolkitApp )
+        toolkitApp->abort();
+
+    toolkitApp = app;
+}
+
+const QList<SimApplication *> SimRules::getSimApps()
+{
+    return simApps;
 }
 
 void SimRules::switchTo(const QString& name)
@@ -923,9 +945,10 @@ bool SimRules::simCsimOk( const QByteArray& payload )
 
     if ( toolkitApp ) {
         QByteArray cmd = toolkitApp->fetch();
-        if ( !cmd.isEmpty() )
+        if ( !cmd.isEmpty() ) {
             sw1 = 0x91;
             sw2 = cmd.size();
+        }
     }
 
     resp += sw1;
@@ -938,16 +961,27 @@ bool SimRules::simCsimOk( const QByteArray& payload )
 
 bool SimRules::simCommand( const QString& cmd )
 {
-    /* Process SIM toolkit begin and end commands by forcing the
-     * app back to the main menu.  */
-    if ( cmd == "AT*TSTB" || cmd == "AT*TSTE" ) {
-        if ( !toolkitApp ) {
+    // 3GPP Terminal Response Command
+    if ( cmd.startsWith("AT+CUSATT=") ) {
+        int start = cmd.indexOf( QChar('=') ) + 1;
+        QByteArray response = QAtUtils::fromHex( cmd.mid(start) );
+        QSimTerminalResponse resp = QSimTerminalResponse::fromPdu( response );
+
+        if ( !toolkitApp || !toolkitApp->response( resp ) )
             respond( "ERROR" );
-            return true;
-        }
+
+        return true;
+    }
+
+    // 3GPP Envelope command
+    if ( cmd.startsWith("AT+CUSATE=") ) {
+        int start = cmd.indexOf( QChar('=') ) + 1;
+        QByteArray envelope = QAtUtils::fromHex( cmd.mid(start) );
+        QSimEnvelope env = QSimEnvelope::fromPdu( envelope );
+
+        if (!toolkitApp || !toolkitApp->envelope( env ) )
+            respond( "ERROR" );
 
-        respond( "OK" );
-        toolkitApp->abort();
         return true;
     }
 
@@ -1012,11 +1046,13 @@ bool SimRules::simCommand( const QString& cmd )
         QSimTerminalResponse resp =
             QSimTerminalResponse::fromPdu( param.mid(5) );
 
-        if ( toolkitApp->response( resp ) )
-            return simCsimOk( QByteArray() );
+        /* Incase of successful case, response is sent inside
+         * the SimApplication::response function. response function
+         * also handles the notification of new command
+         */
+        if ( !toolkitApp->response( resp ) )
+            respond( "+CSIM: 4,6F00\\n\\nOK" );
 
-        /* Response to the wrong type of command. */
-        respond( "+CSIM: 4,6F00\\n\\nOK" );
         return true;
     } else if ( param.length() >= 5 && param[1] == (char)0x2c &&
                     param[4] == (char)0x10 && param.size() >= 21 ) {
@@ -1483,13 +1519,6 @@ SimPhoneBook *SimRules::currentPB() const
         return 0;
 }
 
-void SimRules::setReturnError( const QString &error, uint repeat )
-{
-    return_error_string = error;
-    return_error_count = repeat;
-}
-
-
 int SimRules::newCall()
 {
     int id;
@@ -1564,15 +1593,8 @@ QString expandEscapes( const QString& data, bool eol )
 void SimRules::respond( const QString& resp, int delay, bool eol )
 {
     QString r = expand( resp );
-    if (return_error_string != "") {
-        r = return_error_string;
-        if (return_error_count > 0)
-            return_error_count--;
-        if (return_error_count == 0)
-            return_error_string = "";
-    }
-
     QByteArray escaped = expandEscapes( r, eol ).toUtf8();
+
     if ( !delay ) {
         writeChatData(escaped.data(), escaped.length());
         flush();
@@ -1588,7 +1610,12 @@ void SimRules::respond( const QString& resp, int delay, bool eol )
 
 void SimRules::proactiveCommandNotify( const QByteArray& cmd )
 {
-    unsolicited( "*TCMD: " + QString::number( cmd.size() ) );
+    unsolicited( "+CUSATP: " + QAtUtils::toHex( cmd ) );
+}
+
+void SimRules::modemHandledCommandNotify( const QByteArray& cmd )
+{
+    unsolicited( "*HCMD: " + QAtUtils::toHex( cmd ) );
 }
 
 void SimRules::callControlEventNotify( const QSimControlEvent& evt )
